@@ -280,19 +280,26 @@ class BaseAdapter(with_metaclass(AdapterMeta, ConnectionPool)):
                 #: additional parsing for 'id' fields
                 if ft == "id" and not cacheable:
                     self._add_operators_to_parsed_row(value, table, colset)
-                    self._add_reference_sets_to_parsed_row(
-                        value, table, tablename, colset
-                    )
+                    #: table may be 'nested_select' which doesn't have '_referenced_by'
+                    if hasattr(table, "_referenced_by"):
+                        self._add_reference_sets_to_parsed_row(
+                            value, table, tablename, colset
+                        )
             #: otherwise we set the value in extras
             else:
-                value = self.parse_value(
-                    value, fields[j]._itype, fields[j].type, blob_decode
+                #: fields[j] may be None if only 'colnames' was specified in db.executesql()
+                f_itype, ftype = (
+                    fields[j] and [fields[j]._itype, fields[j].type] or [None, None]
                 )
+                value = self.parse_value(value, f_itype, ftype, blob_decode)
                 extras[colname] = value
-                new_column_match = self._regex_select_as_parser(colname)
-                if new_column_match is not None:
-                    new_column_name = new_column_match.group(1)
-                    new_row[new_column_name] = value
+                if not fields[j]:
+                    new_row[colname] = value
+                else:
+                    new_column_match = self._regex_select_as_parser(colname)
+                    if new_column_match is not None:
+                        new_column_name = new_column_match.group(1)
+                        new_row[new_column_name] = value
         #: add extras if needed (eg. operations results)
         if extras:
             new_row["_extra"] = extras
